var Case = {};

/** the name of the form (set when the BODY loads) */
Case.FORM_NAME = null;
Case.OBJECT_TYPE = null;
Case.KEY = null;
Case.ID = null;
Case.EDITABLE = false;
Case.REFERRER = null;
Case.TITLE = null;
Case.SAVED = false;

Case.getForm = function() {
   return document.getElementById(Case.FORM_NAME);
};

Case.submitForm = function(valueMap)
{
   var form = Case.getForm();
   App.handleFileUploaderForSubmit(form);
   FormUtil.submitForm(form, valueMap);
};

Case.save = function(form)
{
   if (!form)
   {
      form = Case.getForm();
   }
   
   Case.handleDefaultSubmit(form);
   form.submit();
};

Case.saveNew = function()
{
   var form = Case.getForm();
   Case.handleNewDefaultSubmit(form);
   App.handleFileUploaderForSubmit(form);
   form.submit();
};

Case.handleDefaultSubmit = function(form)
{
   App.protectPage();
   App.handleFileUploaderForSubmit(form);
   FormUtil.updateForm(form, {command: "save"});
};

Case.handleNewDefaultSubmit = function(form)
{
   form.target = "_top";
   var formValueMap = {command: "save", isNewEntity: "true"};
   FormUtil.updateForm(form, formValueMap);
};

Case.comments = function() {
   CaseCommon.comments(Case.OBJECT_TYPE, Case.KEY, Case.ID);
};

Case.attachments = function() {
   CaseCommon.attachments(Case.OBJECT_TYPE, Case.KEY, Case.ID);
};

Case.handleReassignDefaultSubmit = function(form)
{
   FormUtil.updateForm(form, {command :"reassign"});
};

Case.reassignSubmit = function(form)
{
   var form = Case.getForm();
   Case.handleReassignDefaultSubmit(form);
   App.handleFileUploaderForSubmit(form);
   form.submit();
};

Case.reassignCancel = function()
{
   window.top.require(['sas-cpb/dialog/Modal'], function(Modal) {
      Modal.hideActiveModal();
   });
};

Case.closeReassignDialog = function()
{
   if (window.parent.Case !== undefined)
   {
      window.parent.Case.submitForm();
   }
   else if (window.parent.Search)
   {
      window.parent.Search.refresh();
   }
};

/**
 * This function is called when the page loads.
 */
Case.handleOnLoad = function()
{
   require(['sas-ecm/shared/Breadcrumb'], function(Breadcrumb) {
      var onload = BrowserUtil.getRequestParameter("onload");
      var gridName = BrowserUtil.getRequestParameter("gridName");
      var relDetailsRequired = BrowserUtil.getRequestParameter('relDetailsRequired') || false;
      var qs = null;

      if (App.HAS_ERRORS && Case.SAVED)
      {
         qs = "&key=" + Case.KEY;
      }
      Breadcrumb.add(Case.REFERRER, Case.TITLE, qs);

      Breadcrumb.logTrail();

      if (!onload || App.HAS_ERRORS)
      {
         return;
      }
   
      if (onload == "newParty")
      {
         Case.showNewPartyDialog(Case.UI_CONTEXT_ID, false, gridName, relDetailsRequired);
      }
   
      if (onload == "newReport")
      {
         Case.showNewReportDialog();
      }
   });
};

/**
 * This method is invoked to open a different URL. If changes have been
 * made to the current page then a "Save Changes" confirmation dialog is
 * shown before redirecting.
 */
Case.redirect = function(redirectURL, extraCallback)
{
   var callbacks = [ Case.saveChangesConfirmationCallback ];
   if (extraCallback) {
      // unshift makes it first. this is important because the saveConfirmation will change the window.location.
      callbacks.unshift(extraCallback);
   }
   var args = {redirectURL: redirectURL};
   SaveChangesConfirmation.confirmSaveChanges(callbacks, args);
};

Case.logoff = function()
{
   Case.redirect("Logoff");
};

Case.saveChangesConfirmationCallback = function(saveChangesConfirmation)
{
   App.protectPage();
   
   var resultCode = saveChangesConfirmation.resultCode;
   
   if (resultCode == SaveChangesConfirmation.ResultCode.YES)
   {
      var separator = "?";
      if (saveChangesConfirmation.args.redirectURL.indexOf("?") > -1)
      {
         separator = "&";
      }
      var redirectURL = saveChangesConfirmation.args.redirectURL + separator + "caseSaved=" + Case.KEY;
      
      /* save the case but provide a redirect URL */
      var form = Case.getForm();
      FormUtil.updateForm(form, {redirectURL: redirectURL});
      Case.save(form);
   }
   else if ((resultCode == SaveChangesConfirmation.ResultCode.NO) ||
            (resultCode == SaveChangesConfirmation.ResultCode.NOT_DIRTY))
   {
      /* user does not want to save or the page is not dirty */
      window.location = saveChangesConfirmation.args.redirectURL;
   }
};

Case.newParty = function(uiContextID, caseKey, fromModal, gridName, relDetailsRequired)
{
   Case.KEY = caseKey;
   var args = {uiContextID: uiContextID, fromModal: fromModal };
   if (gridName) {
      args.gridName = gridName;
   }
   if (relDetailsRequired) {
      args.relDetailsRequired = relDetailsRequired;
   }
   if (fromModal) {
      parent.SaveChangesConfirmation.confirmSaveChanges(Case.newPartySaveChangesConfirmationCallback, args);
   }
   else  {
      SaveChangesConfirmation.confirmSaveChanges(Case.newPartySaveChangesConfirmationCallback, args);
   }
};

Case.newPartySaveChangesConfirmationCallback = function(saveChangesConfirmation)
{
   var resultCode = saveChangesConfirmation.resultCode;
   var fromModal = saveChangesConfirmation.args.fromModal;
   var gridName = saveChangesConfirmation.args.gridName;
   var relDetailsRequired = saveChangesConfirmation.args.relDetailsRequired || false;
   
   if (resultCode == SaveChangesConfirmation.ResultCode.YES)
   {
      App.protectPage();
      
      /* set parameters that will cause new case dialog to appear after save */
      if (fromModal)
      {
         var form = parent.Case.getForm();
         var params = {onload: "newParty"};
         if (gridName) {
            params.gridName = gridName;
         }
         if (relDetailsRequired) {
            params.relDetailsRequired = relDetailsRequired;
         }
         parent.FormUtil.updateForm(form, params);
         
         // save the case
         parent.Case.save(form);
      }
      else
      {
         var form = Case.getForm();
         var params = {onload: "newParty"};
         if (gridName) {
            params.gridName = gridName;
         }
         if (relDetailsRequired) {
            params.relDetailsRequired = relDetailsRequired;
         }
         FormUtil.updateForm(form, params);
         
         // save the case
         Case.save(form);
      }
   }
   else if ((resultCode == SaveChangesConfirmation.ResultCode.NO) ||
            (resultCode == SaveChangesConfirmation.ResultCode.NOT_DIRTY))
   {
      Case.showNewPartyDialog(saveChangesConfirmation.args.uiContextID, fromModal, gridName, relDetailsRequired);
   }
};

Case.showNewPartyDialog = function(uiContextID, fromModal, gridName, relDetailsRequired)
{
   var url = Global.CONTEXT_PATH + "/Party.do?command=newEntity&forCaseKey=" + Case.KEY + "&parentUIContextID="+ uiContextID;
   if (gridName) {
      url += "&gridName=" + gridName;
   }
   if (relDetailsRequired) {
      url += '&relDetailsRequired=true';
   }
   window.top.require(['sas-cpb/dialog/Modal'], function(Modal) {
      if (fromModal) {
         Modal.updateActiveModal(
            url,
            window.top.AppProperties[gridName + '.dialog.addNew.title.txt']
         );
      }
      else {
         Modal.show({
            url: url,
            width: 750, height: 450,
            title: AppProperties[gridName + '.dialog.addNew.title.txt']
         });
      }
   });
};

Case.showNewReportDialog = function()
{
   var url = Global.CONTEXT_PATH + "/Report.do?command=newEntity";
   window.top.require(['sas-cpb/dialog/Modal'], function(Modal) {
      Modal.show({
         url: url,
         width: 500, height: 200,
         title: AppProperties["newReport.title.txt"]});
   });
};


Case.print = function()
{
   CaseCommon.print(Case.KEY);
};

Case.printSave = function()
{
   CaseCommon.printSave(Case.KEY, Case.ID);
};
