/*
 * Utility methods for grids.
 */
define(
      "sas-cpb/datagrid/CPBGridUtil", ['sas-cpb/cpb', 'sas-cpb/datagrid/CPBDataGrid', 'sas-cpb/menu/CPBMenu',
       'sas-cpb/datagrid/GridActionImageType', 'sas-cpb/datagrid/GridActionOutputType',
       'sas-cpb/messages/Messages', 'dojo/_base/array', 'dojo/request/xhr', 'dojo/dom-construct',
       'dijit/form/Button', 'dijit/form/DropDownButton', 'dijit/DropDownMenu', 'dijit/MenuItem',
       'dojo/domReady!'],
      function(cpb, CPBDataGrid, CPBMenu,
               GridActionImageType, GridActionOutputType,
               Messages, array, xhr, domConstruct,
               Button, DropDownButton, DropDownMenu, MenuItem) {

var CPBGridUtil = {};


// Analog of DataGridHelper
CPBGridUtil.getFormattedUINameFromID = function(id) {
   return id.replace(/\./g, '__');
};

//Analog of DataGridHelper
CPBGridUtil.getIDFromFormattedUIName = function(name) {
   return name.replace(/__/g, '.');
};

CPBGridUtil.getURL = function(gridName, uiContextID, cacheDataInUIContext) {
   var action = !cacheDataInUIContext ? 'allRowData' : 'rowData';
   return 'controller/datagrid/' + action + '/' + gridName + '/' + uiContextID + '.json';
};

var _getActionFormattedUIName = function(gridName, action) {
   return action.id.replace(/\./g, '__') + '_' + gridName.replace(/\./g, '__');
};

// Programmatically add a style to reference an image by URL.  This is here for backwards
// compatibility with the deprecated use of <image> tags with the GetThemedImageURL() function
// from ECM 3.2. Documentation now advises to use CSS class names instead of the old way. 
var _createCssClassForImage = function(url) {
   
   var lastSlash = url.lastIndexOf('/');
   var className = url.substring(lastSlash+1).replace('.',  '_').trim();
   
   var style = domConstruct.create('style', { type: 'text/css' }, document.body);
   style.innerHTML = '.' + className + ' { background-image: url(' + url + '); ' +
                     'background-repeat: no-repeat; width: 18px; height: 18px; ' +
                     'text-align: center; }';

   return className;
};

// summary:
//    Creates a column model suitable for a dgrid-based CPB component.  Note that this may mutate the
//    columnModel passed in.
CPBGridUtil.createGridColumns = function(formattedGridName, columnModel, rowActions, lazyRowActionsUrl, hasSelectorColumn) {

   for (var i=0; i<columnModel.length; i++) {
      var colModel = columnModel[i];
      if (typeof colModel.renderCell === 'string') { // Convert function names to functions
         colModel.renderCell = window[colModel.renderCell];
      }
   }
   
   // If there are row-level actions, prepend a column for a dropdown menu for them
   if (lazyRowActionsUrl || (rowActions && rowActions.length > 0)) {
      var renderCellFunction = function(rowData, data, node, options) {
         
         // Note: rowData.rowIndex is typically undefined, but is set for GEG's
         var rowIndex = rowData.rowIndex;

         var button = new DropDownButton({
            showLabel: false
         });

         if (!lazyRowActionsUrl) {
            button.dropDown = _createStandardActionMenu(formattedGridName, button, rowActions, rowIndex, rowData);
         }
         else {
            _enableLazyActionMenu(formattedGridName, button, rowActions, rowIndex, rowData, lazyRowActionsUrl);
         }
         
         node.appendChild(button.domNode);

      };
      
      var column = { field: 'actionMenuColumn', label: ' ', resizable: false, unhidable: true, sortable: false, reorderable: false, renderCell: renderCellFunction };
      columnModel.splice(hasSelectorColumn ? 1 : 0, 0, column);
      
   }

   return columnModel;
};

// summary:
//    Creates a row-level action menu when all grid actions are known at render time.
var _createStandardActionMenu = function(gridName, parent, rowActions, row, rowData) {
   var actionMenu = new DropDownMenu({ style: 'display: none;', autoFocus: true });
   var menuItems = _createRowActionsMenuItems(gridName, rowActions, row, rowData);
   for (var i = 0; i < menuItems.length; i++)  {
    actionMenu.addChild(menuItems[i]);
   }
   return actionMenu;
};

// summary:
//    Creates a row-level action menu that lazily fetches (some of) its child menu items when it's first displayed.
var _enableLazyActionMenu = function(gridName, parent, rowActions, row, rowData, lazyActionsUrl) {
   var grid = CPBDataGrid.getGrid(gridName);
   var queryString = 'key=' + rowData[grid.cpb_selectedKeyField];
   var questionMark = lazyActionsUrl.indexOf('?');
   if (questionMark>-1) {
      queryString += '&' + lazyActionsUrl.substring(questionMark+1);
      lazyActionsUrl = lazyActionsUrl.substring(0, questionMark);
   }
   
   var args = {
      query: queryString,
      handleAs: 'json',
      sync: true
   };

   parent.loadDropDown = function(callback) {
      xhr(lazyActionsUrl, args).then(function(data) {
         // Add laziliy-fetched menu items first.
         parent.dropDown = CPBMenu.buildMenu(parent, data.menuItemList);
         // Add any user-defined menu items next.
         var userDefinedItems = _createRowActionsMenuItems(gridName, rowActions, row, rowData);
         for (var i=0; i<userDefinedItems.length; i++) {
            parent.dropDown.addChild(userDefinedItems[i]);
         }
         if (callback) { callback(); }
      }, function(err) {
         grid.cpb_showMessage(AppProperties['table.error.loadingActionMenu.txt'], 'error');
         cpb.consoleLog(err, 'error');
      });
      //this.inherited(arguments);
   };

};

var _createRowActionsMenuItems = function(gridName, rowActions, row, rowData) {
   
   var items = [];
   
   if (rowActions && rowActions.length) {
      for (var i = 0; i < rowActions.length; i++) {
         var action = rowActions[i];
         items.push(new MenuItem({
            label: action.label,
            disabled: action.disabled,
            iconClass: action.iconClass,
            onClick: function(delegate) {
               return function()  {
                  // Set some variables useful for JS actions
                  var grid = CPBDataGrid.getGrid(gridName);
                  actionData = {
                     rowIndex: row, // For backwards compatibility, not set anymore with dgrid
                     gridName: gridName,
                     _grid: grid,
                     checkedRowKeys: CPBDataGrid.getSelectedRowKeys(grid),
                     selectedRowKey: rowData[grid.cpb_selectedKeyField],
                     selectedRowData: rowData
                  };
                  delegate.onClick();
               };
            }(action)
         }));
      }
   }
   
   return items;
};

// summary:
//    Returns an array of grid-level actions, an array of row-level actions, and a row of refresh
//    listeners for a grid, from a model returned from the server.
CPBGridUtil.createGridActions = function(gridName, gridActionsModel) {
   
   var formattedGridName = CPBGridUtil.getFormattedUINameFromID(gridName);
   
   var hasInlineGetActions, hasInlinePostActions;
   var gridActions = [];
   var rowActions = [];
   var refreshListeners = [];
   
   array.forEach(gridActionsModel, function(action) {
      
      var label = action.label;
      var tip = action.tip;
      var hasTableLevelActions = false;
      action.visible = action.visible || true;
      
      if (action.visible) {
         var actionVarName = _getActionFormattedUIName(formattedGridName, action);
         if (action.output===GridActionOutputType.INLINE) {
            if ('get'===action.requestType) {
               hasInlineGetActions = true;
            }
            else if ('post'===action.requestType) {
               hasInlinePostActions = true;
            }
         }
         
         // Create the button
         var requiresSelection = 'select'===action.renderType;
         var disabledOnRender = action.disabledOnRender || false;
         var imageType = GridActionImageType.get(action.imageType);
         
         var buttonArgs = {
            id: actionVarName,
            sas_selectionButton: requiresSelection,
            disabled: requiresSelection || disabledOnRender,
            showLabel: (label!=null)
         };
         if (label) { buttonArgs.label = label; }
         if (tip) { buttonArgs.title = tip; }
         if (action.image) {
            var className;
            if (imageType===GridActionImageType.URL) {
               className = _createCssClassForImage(action.image);
            }
            else { // GridActionImageType.CLASS
               className = action.image;
            }
            buttonArgs.iconClass = className;
         }
         var actionButton = new Button(buttonArgs);
         
         actionButton.onClick = function() {
            var selectedKeys = CPBDataGrid.getSelectedRowKeys(formattedGridName);
            var actionId = action.id;
            var callback = action.callback;
            // Row-level JS actions created actionData already; grid-level JS actions must create it here.
            if (action.output===GridActionOutputType.JAVASCRIPT && 'row'!==action.renderType) {
               curGrid = CPBDataGrid.getGrid(formattedGridName);
               actionData = {
                  rowIndex: null, // No row is selected; this is a table-level action
                  gridName: curGrid.cpb_name,
                  _grid: curGrid,
                  checkedRowKeys: CPBDataGrid.getSelectedRowKeys(formattedGridName),
                  selectedRowKey: null,
                  selectedRowData: null
               };
            }
            if (action.onClick) { // Actions defined in JS in grid's AMD module
               action.onClick(formattedGridName, selectedKeys, callback);
            }
            else { // Actions defined in uidef XML, or in Java
               // These callbacks can't be local variables, or they'll be optimized out since they're not
               // referenced directly.
               var method = '_' + action.jsOnClickHandler + 'GridActionHandler';
               // Actions of type === 'row' had an actionData created in their onClick handler with selectedRowKey set.
               // Row-level actions should only submit that row's key, not all selected rows' keys.
               var keys = ('row'===action.renderType && actionData.selectedRowKey) ? actionData.selectedRowKey : selectedKeys;
               CPBGridUtil[method](actionId, formattedGridName, keys, callback);
            }
         };
         
         // Determine if the button goes above the table or on the row.
         if ('row'===action.renderType) {
            rowActions.push(actionButton);
         }
         else {
            // register the onrefresh method if it exists
            if (action.onRefresh) {
               actionButton.onrefreshListener = function() {
                  cpb.consoleLog('onrefreshListener called for action: ' + action.id);
                  eval(action.onRefresh);
               };
               refreshListeners.push(actionButton);
            }
            gridActions.push(actionButton);
            hasTableLevelActions = true;
         }
      }
      
   }); // array.forEach(gridActionsModel, function(action)

   return { gridActions: gridActions, rowActions: rowActions, refreshListeners: refreshListeners };
};

// summary:
//    Executes grid actions whose payloads are URLs to GET or POST, but don't refresh the grid.
var _ignoreImpl = function(formMethod, actionId, gridVarName, selectedKeys, callback) {
   CPBService.getGridActionURLData(actionId, cpb.uiContextId, cpb.screenId, cpb.getFormParams(),
      function(data) {

         if (data.validationErrors) {
            showValidationErrors(data.validationErrors);
            return;
         }
         var grid = CPBDataGrid.getGrid(gridVarName);

         var url = new URL(data.url);

         // Create a form to submit the URL, adding our "standard" parameters.
         var form = domConstruct.create('form', { method: formMethod, action: url.path, style: 'display: none' }, document.body);
         domConstruct.create('input', { type: 'hidden', name: 'keys', value: selectedKeys.join(',') }, form);
         domConstruct.create('input', { type: 'hidden', name: '__cpb_uiContextId', value: cpb.uiContextId }, form);
         domConstruct.create('input', { type: 'hidden', name: '__cpb_gridName', value: gridVarName }, form);
         domConstruct.create('input', { type: 'hidden', name: '__cpb_sort', value: grid.get('sortStr') }, form);
         //form.addEventListener('submit', function(e) { alert('hi'); return false; });

         // Add parameters passed to the grid action in XML.
         var params = url.getQueryString().getParameters();
         for (var paramName in params) {
            if (params.hasOwnProperty(paramName)) {
               domConstruct.create('input', { type: 'hidden', name: paramName, value: params[paramName] }, form);
            }
         }

         form.submit();
         domConstruct.destroy(form);
      }
   );
};

// summary:
//    Executes a GET request from a grid action.
var _gridAjaxGetImpl = function(grid, url, selectedKeys, callback, sync) {
   
   if (!sync) {
      sync = false;
   }
   
   if (url && url.indexOf('?')!==0) {
      
      var combinedKeys = selectedKeys.join(",");
   
      cpb.consoleLog("url get :: " + url);
      cpb.consoleLog("click :: id == " + combinedKeys);
      var appendChar = url.indexOf('?')>0 ? '&' : '?';
      url += appendChar + "keys=" + combinedKeys;

      url += '&__cpb_uiContextId=' + cpb.uiContextId + '&__cpb_gridName=' + grid.cpb_name;
      url += '&__cpb_sort=' + grid.get('sortStr');

      dojo.xhrGet({
         url: url,
         sync: sync,
         load: function(data, ioargs) {
            var result = { data: data, statusCode: ioargs.xhr.status, gridName: grid.cpb_name, _grid: grid, _ioargs: ioargs };
            if (callback!=null) {
               _gridCallbackHandler(grid, callback, result);
            }
            _gridActionAjaxSucceed(grid, data, ioargs);
         },
         error: function(error, ioargs) {
            _gridActionAjaxFail(grid, error, ioargs);
         }
      });
      
   }
   
   // Empty URLs => just refresh the grid.
   else {
      grid.clearSelection();
      CPBDataGrid.refreshServerStore(grid.cpb_storeName);
      if (callback!=null) {
         _gridCallbackHandler(grid, callback);
      }
   }
   
};

// summary:
//    Executes a POST request from a grid action.
var _gridAjaxPostImpl = function(grid, url, queryString, selectedKeys, callback, sync) {
   
   if (!sync) {
      sync = false;
   }
   
   if (url) {
      
      cpb.consoleLog("url post :: " + url);
      cpb.consoleLog("queryString :: " + queryString);
      cpb.consoleLog("click :: selectedKeys == " + selectedKeys);
      var params = QueryString.create(queryString);
      for (var i = 0; i < selectedKeys.length; i++) {
         params.addParameter('keys', selectedKeys[i]);
      }
      
      params.addParameter('__cpb_uiContextId', cpb.uiContextId);
      params.addParameter('__cpb_gridName', grid.cpb_name);
      params.addParameter('__cpb_sort', grid.get('sortStr'));

      dojo.xhrPost({
         url: url,
         postData: params.toUnencodedString(),
         sync: sync,
         load: function(data, ioargs) {
            var result = { data: data, statusCode: ioargs.xhr.status, gridName: grid.cpb_name, _grid: grid, _ioargs: ioargs };
            if (callback!=null) {
               _gridCallbackHandler(grid, callback, result);
            }
            _gridActionAjaxSucceed(grid, data, ioargs);
         },
         error: function(error, ioargs) {
            _gridActionAjaxFail(grid, error, ioargs);
         }
      });
      
   }
   
   // Empty URLs => just refresh the grid.
   else {
      grid.clearSelection();
      CPBDataGrid.refreshServerStore(grid.cpb_storeName);
      if (callback!=null) {
         _gridCallbackHandler(grid, callback);
      }
   }
};

// summary:
//    Called when a grid action's GET or POST request is successful.
var _gridActionAjaxSucceed = function(grid, data, ioargs) {
   var statusCode = ioargs.xhr.status; // should be 200
   cpb.consoleLog('Grid ' + grid.cpb_name + ' ajax call succeeded with status code: ' + statusCode);
   
   grid.clearSelection();
   CPBDataGrid.refreshServerStore(grid.cpb_storeName);
   //grid.cpb_showMessage("Operation successful", 'status');
};

// summary:
//    Called when a grid action's GET or POST request fails
var _gridActionAjaxFail = function(grid, error, ioargs) {
   cpb.consoleLog('Grid ' + grid.cpb_name + ' ajax call failed with message: ' + error, 'error');
   CPBDataGrid.hideMaskForAllGridByStore(grid.cpb_storeName);
   grid.cpb_showMessage("Error occurred (status " + ioargs.xhr.status + "): '" + error + "'.", 'error');
};

// summary:
//    Executes a callback for a grid action that was specified in a UI definition.
var _gridCallbackHandler = function(grid, callback, result) {
   var success = true;
   try {
      eval(callback);
   } catch (err) {
      var msg = AppProperties['errors.datagrid.errorRunningCallback.fmt.txt'];
      // TODO: substitute formatted grid name into message above
      cpb.consoleLog(msg + ":\n" + err, 'error');
      grid.cpb_showMessage(msg, 'error');
      success = false;
   }
   return success;
};

// summary:
//    Executes grid actions whose payloads are JavaScript.
CPBGridUtil._javaScriptGridActionHandler = function(actionId, gridVarName, selectedKeys, callback) {

   var grid = CPBDataGrid.getGrid(gridVarName);
   CPBDataGrid.showMaskForAllGridByStore(grid.cpb_storeName);

   CPBService.getGridActionURLData(actionId, cpb.uiContextId, cpb.screenId, cpb.getFormParams(),
      function(data) {
         if (data.validationErrors) {
            CPBDataGrid.hideMaskForAllGridByStore(grid.cpb_storeName);
            showValidationErrors(data.validationErrors);
            return;
         }
         try {
            eval(data.url);
         } catch (err) {
            var message = 'Could not eval JS GridAction: ' + err;
            cpb.consoleLog(message, 'error');
            grid.cpb_showMessage(message, 'error');
         } finally {
            CPBDataGrid.hideMaskForAllGridByStore(grid.cpb_storeName);
         }
      }
   );
};

// summary:
//    Executes grid actions whose payloads are URLs to POST, but don't refresh the grid.
CPBGridUtil._ignorePostGridActionHandler = function(actionId, gridVarName, selectedKeys, callback) {
   _ignoreImpl('POST', actionId, gridVarName, selectedKeys, callback);
};

// summary:
//    Executes grid actions whose payloads are URLs to GET, but don't refresh the grid.
CPBGridUtil._ignoreGetGridActionHandler = function(actionId, gridVarName, selectedKeys, callback) {
   _ignoreImpl('GET', actionId, gridVarName, selectedKeys, callback);
};

// summary:
//    Executes grid actions whose payloads are URLs to POST.
CPBGridUtil._inlinePostGridActionHandler = function(actionId, gridVarName, selectedKeys, callback) {

   var grid = CPBDataGrid.getGrid(gridVarName);
   CPBDataGrid.showMaskForAllGridByStore(grid.cpb_storeName);

   CPBService.getGridActionURLData(actionId, cpb.uiContextId, cpb.screenId, cpb.getFormParams(),
      function(data) {
         if (data.validationErrors) {
            CPBDataGrid.hideMaskForAllGridByStore(grid.cpb_storeName);
            showValidationErrors(data.validationErrors);
            return;
         }
         var url = new URL(data.url);
         // use QueryString's toUnencodedString() instead of toString(), since the QueryString was already encoded in the Java tier.
         _gridAjaxPostImpl(grid, url.path, url.queryString? url.queryString.toUnencodedString() : null, selectedKeys, callback);
      }
   );
};

// summary:
//    Executes grid actions whose payloads are URLs to GET.
CPBGridUtil._inlineGetGridActionHandler = function(actionId, gridVarName, selectedKeys, callback) {

   var grid = CPBDataGrid.getGrid(gridVarName);
   CPBDataGrid.showMaskForAllGridByStore(grid.cpb_storeName);

   CPBService.getGridActionURLData(actionId, cpb.uiContextId, cpb.screenId, cpb.getFormParams(),
      function(data) {
         if (data.validationErrors) {
            CPBDataGrid.hideMaskForAllGridByStore(grid.cpb_storeName);
            showValidationErrors(data.validationErrors);
            return;
         }
         _gridAjaxGetImpl(grid, data.url, selectedKeys, callback);
      }
   );
};

// summary:
//    Executes grid actions whose payloads are URLs to open in a new window.
CPBGridUtil._newWindowGridActionHandler = function(actionId, gridVarName, selectedKeys, callback) {

   var grid = CPBDataGrid.getGrid(gridVarName);
   CPBDataGrid.showMaskForAllGridByStore(grid.cpb_storeName);

   CPBService.getGridActionURLData(actionId, cpb.uiContextId, cpb.screenId, cpb.getFormParams(),
      function(data) {
         if (data.validationErrors) {
            CPBDataGrid.hideMaskForAllGridByStore(grid.cpb_storeName);
            showValidationErrors(data.validationErrors);
            return;
         }
         var url = data.url + (data.url.indexOf("?")===-1 ? "?" : "&") + "keys=" + selectedKeys;
         window.open(url, '_blank');
         CPBDataGrid.hideMaskForAllGridByStore(grid.cpb_storeName);
      }
   );
};

// summary:
//    Executes grid actions whose payloads are URLs to redirect the current window to.
CPBGridUtil._windowGridActionHandler = function(actionId, gridVarName, selectedKeys, callback) {

   var grid = CPBDataGrid.getGrid(gridVarName);
   CPBDataGrid.showMaskForAllGridByStore(grid.cpb_storeName);

   CPBService.getGridActionURLData(actionId, cpb.uiContextId, cpb.screenId, cpb.getFormParams(),
      function(data) {
         if (data.validationErrors) {
            CPBDataGrid.hideMaskForAllGridByStore(grid.cpb_storeName);
            showValidationErrors(data.validationErrors);
            return;
         }
         var url = data.url + (data.url.indexOf("?")===-1 ? "?" : "&") + "keys=" + selectedKeys + "&__cpb_uiContextId=" + cpb.uiContextId;
         if (cpb.redirectHandler != null) {
            cpb.redirectHandler(url);
         }
         else {
            window.location.href = url;
         }
         // No need to hide the grid mask, as the new page will open in the current window.
      }
   );
};

var showValidationErrors = function(errors) {
   Messages.showError(errors); // Arrays are OK as a parameter here
};

return CPBGridUtil;

});
