var Incident = {};

/** the name of the form (set when the BODY loads) */
Incident.FORM_NAME = null;
Incident.OBJECT_TYPE = null;
Incident.KEY = null;
Incident.ID = null;
Incident.EDITABLE = false;
Incident.UI_CONTEXT_ID = null;
Incident.REFERRER = null;
Incident.TITLE = null;
Incident.SAVED = false;

Incident.getForm = function()
{
   return document.getElementById(Incident.FORM_NAME);
};

Incident.submitForm = function(valueMap)
{
   var form = Incident.getForm();
   App.handleFileUploaderForSubmit(form);
   FormUtil.submitForm(form, valueMap);
};

Incident.save = function(form)
{
   if (!form)
   {
      form = Incident.getForm();
   }
   
   Incident.handleDefaultSubmit(form);
   form.submit();
};

Incident.saveNew = function()
{
   var form = Incident.getForm();
   Incident.handleNewDefaultSubmit(form);
   App.handleFileUploaderForSubmit(form);
   form.submit();
};

Incident.handleDefaultSubmit = function(form)
{
   App.protectPage();
   App.handleFileUploaderForSubmit(form);
   FormUtil.updateForm(form, {command: "save"});
};

Incident.handleNewDefaultSubmit = function(form)
{
   form.target = "_top";
   FormUtil.updateForm(form, {command: "save", isNewEntity: "true"});
};

Incident.relatedCases = function()
{
   require(['sas-cpb/dialog/Modal'], function(Modal) {
      var form = Incident.getForm();
      
      var url = Global.CONTEXT_PATH + "/AddIncidentToCase.do" +
         "?key=" + form.elements["key"].value +
         "&uiContextId=" + form.elements["uiContextId"].value +
         "&windowLayout=" + WindowLayout.MODAL_DIALOG;
      var title = AppProperties["incident.relatedCases.title.txt"] + 
         "&nbsp;&#149;&nbsp;" + 
         AppProperties["incident.title.txt"] + "&nbsp;" + Incident.ID;
      Modal.show({
         url: url,
         width: 900, height: 500,
         title: title});
   });
};

Incident.refreshCaseLink = function(elementID)
{
   var el = document.getElementById(elementID);
   if (el && el != null)
   {
      PleaseWait.show(el);
         
      IncidentCaseLinkComponentService.refreshLink(
               Incident.UI_CONTEXT_ID, 
               function(html) {
                  el.innerHTML = html;
               }
      );
   }
   
   if (window.SaveChangesConfirmation)
   {
      SaveChangesConfirmation.setPageDirty();
   }
};

Incident.deleteIncident = function()
{
   Incident.submitForm({command: "deleteIncident"});
};

Incident.comments = function()
{
   Comments.showDialog(Incident.KEY, Incident.OBJECT_TYPE, AppProperties["incident.title.txt"] + "&nbsp;" + Incident.ID);
};

Incident.attachments = function()
{
   require(['sas-cpb/dialog/Modal', 'sas-cpb/util/NLSUtil'], function(Modal, NLSUtil) {
      var url = Global.CONTEXT_PATH + "/Attachments.do?command=display&key=" + Incident.KEY + "&objType=" + Incident.OBJECT_TYPE;
      var title =
            AppProperties["attachments.title.txt"] +
            "&nbsp;&#149;&nbsp;" +
            AppProperties["incident.title.txt"] +
            "&nbsp;" +
            NLSUtil.wrapInLTRForcingMarkup(Incident.ID);
      Modal.show({url: url, title: title});
   });
};

Incident.handleFormOnSubmit = function(form)
{
   App.handleFileUploaderForSubmit(form);
   return true;
};

Incident.newCase = function()
{
   SaveChangesConfirmation.confirmSaveChanges(
         Incident.newCaseSaveChangesConfirmationCallback);
};

Incident.showNewCaseDialog = function(key)
{
   var url = Global.CONTEXT_PATH + "/Case.do?command=newEntity&forIncidentKey=" + key;
   require(['sas-cpb/dialog/Modal'], function(Modal) {
      Modal.show({
         url: url,
         width: 600, height: 400,
         title: AppProperties["incident.newCase.title.txt"]});
   });
};

Incident.newCaseSaveChangesConfirmationCallback = function(saveChangesConfirmation)
{
   var resultCode = saveChangesConfirmation.resultCode;
   
   if (resultCode == SaveChangesConfirmation.ResultCode.YES)
   {
      /* set parameters that will cause new case dialog to appear after save */
      var form = Incident.getForm();
      FormUtil.updateForm(form, {onload: "newCase"});
      
      /* save the incident */
      Incident.save(form);
   }
   else if ((resultCode == SaveChangesConfirmation.ResultCode.NO) ||
            (resultCode == SaveChangesConfirmation.ResultCode.NOT_DIRTY))
   {
      Incident.showNewCaseDialog(Incident.KEY);
   }
};

Incident.logoff = function()
{
   Incident.redirect("Logoff");
};

Incident.redirect = function(redirectURL, extraCallback)
{
   var callbacks = [ Incident.saveChangesConfirmationCallback ];
   if (extraCallback) {
      // unshift makes it first. this is important because the saveConfirmation will change the window.location.
      callbacks.unshift(extraCallback);
   }
   var args = {redirectURL: redirectURL};
   SaveChangesConfirmation.confirmSaveChanges(callbacks, args);
};

Incident.saveChangesConfirmationCallback = function(saveChangesConfirmation)
{
   App.protectPage();
   
   var resultCode = saveChangesConfirmation.resultCode;
   
   if (resultCode == SaveChangesConfirmation.ResultCode.YES)
   {
      var separator = "?";
      if (saveChangesConfirmation.args.redirectURL.indexOf("?") > -1)
      {
         separator = "&";
      }
      var redirectURL = 
            saveChangesConfirmation.args.redirectURL + separator + "incidentSaved=" + Incident.KEY;
      
      /* save the incident but provide a redirect URL */
      var form = Incident.getForm();
      FormUtil.updateForm(form, {redirectURL: redirectURL});
      Incident.save(form);
   }
   else if ((resultCode == SaveChangesConfirmation.ResultCode.NO) ||
            (resultCode == SaveChangesConfirmation.ResultCode.NOT_DIRTY))
   {
      /* user does not want to save or the page is not dirty */
      window.location = saveChangesConfirmation.args.redirectURL;
   }
};

Incident.handleOnLoad = function()
{
   require(['sas-ecm/shared/Breadcrumb'], function(Breadcrumb) {
      var onload = BrowserUtil.getRequestParameter("onload");
      var gridName = BrowserUtil.getRequestParameter("gridName");
      var qs = null;

      if (App.HAS_ERRORS && Incident.SAVED)
      {
         qs = "&key=" + Incident.KEY;
      }
      Breadcrumb.add(Incident.REFERRER, Incident.TITLE, qs);

      if (!onload || App.HAS_ERRORS)
      {
         return;
      }
   
      if (onload == "newCase")
      {
         Incident.showNewCaseDialog(Incident.KEY);
      }
   
      if (onload == "newParty")
      {
         Incident.showNewPartyDialog(Incident.UI_CONTEXT_ID, false, gridName);
      }
   
      if (onload == "newReport")
      {
         Incident.showNewReportDialog();
      }
   });
};

Incident.newParty = function(uiContextId, incidentKey, fromModal, gridName)
{
   Incident.KEY = incidentKey;
   var args = {uiContextId: uiContextId, fromModal: fromModal};
   if (gridName)
   {
      args.gridName = gridName;
   }
   if (fromModal)
   {
      parent.SaveChangesConfirmation.confirmSaveChanges(Incident.newPartySaveChangesConfirmationCallback, args);
   }
   else 
   {
      SaveChangesConfirmation.confirmSaveChanges(Incident.newPartySaveChangesConfirmationCallback, args);
   }
};

Incident.newPartySaveChangesConfirmationCallback = function(saveChangesConfirmation)
{
   var resultCode = saveChangesConfirmation.resultCode;
   var fromModal = saveChangesConfirmation.args.fromModal;
   var gridName = saveChangesConfirmation.args.gridName;
   
   if (resultCode == SaveChangesConfirmation.ResultCode.YES)
   {
      App.protectPage();
      
      /* set parameters that will cause new party dialog to appear after save */
      if (fromModal)
      {
         var form = parent.Incident.getForm();
         var params = {onload: "newParty"};
         if (gridName) {
            params.gridName = gridName;
         }
         parent.FormUtil.updateForm(form, params);
         
         // save the incident
         parent.Incident.save(form);
      }
      else
      {
         var form = Incident.getForm();
         var params = {onload: "newParty"};
         if (gridName) {
            params.gridName = gridName;
         }
         FormUtil.updateForm(form, params);
         
         // save the incident
         Incident.save(form);
      }
   }
   else if ((resultCode == SaveChangesConfirmation.ResultCode.NO) ||
            (resultCode == SaveChangesConfirmation.ResultCode.NOT_DIRTY))
   {
      Incident.showNewPartyDialog(saveChangesConfirmation.args.uiContextId, fromModal, gridName);
   }
};

Incident.showNewPartyDialog = function(uiContextId, fromModal, gridName)
{
   window.top.require(['sas-cpb/dialog/Modal'], function(Modal) {
      var url = Global.CONTEXT_PATH + "/Party.do?command=newEntity&forIncidentKey=" + Incident.KEY + "&parentUIContextID=" + uiContextId;
      if (gridName)
      {
         url += "&gridName=" + gridName;
      }
   
      if (fromModal) {
         Modal.updateActiveModal(url, AppProperties[gridName + '.dialog.addNew.title.txt']);
      }
      else {
         Modal.show({
            url: url,
            width: 850, height: 450,
            title: AppProperties[gridName + '.dialog.addNew.title.txt']});
      }
   });
};

Incident.showNewReportDialog = function()
{
   require(['sas-cpb/dialog/Modal'], function(Modal) {
      var url = Global.CONTEXT_PATH + "/Report.do?command=newEntity";
      Modal.show({
         url: url,
         width: 500, height: 200,
         title: AppProperties["newReport.title.txt"]});
   });
};

Incident.handleReassignDefaultSubmit = function(form)
{
   FormUtil.updateForm(form, {command :"reassign"});
};

Incident.reassignSubmit = function(form)
{
   var form = Incident.getForm();
   Incident.handleReassignDefaultSubmit(form);
   App.handleFileUploaderForSubmit(form);
   form.submit();
};

Incident.reassignCancel = function()
{
   window.top.require(['sas-cpb/dialog/Modal'], function(Modal) {
      Modal.hideActiveModal();
   });
};

Incident.closeReassignDialog = function()
{
   if (window.parent.Incident !== undefined)
   {
      window.parent.Incident.submitForm();
   }
   else if (window.parent.Search)
   {
      window.parent.Search.refresh();
   }
};
